@@ -44,31 +44,59 @@
 
 #include "daemon.h"
 #include "engines.h"
+#include "stack.h"
 #include "notificationdaemon-dbus-glue.h"
 
 #define IMAGE_SIZE 48
 
-struct _NotifyTimeout
-{
-	GTimeVal expiration;
-	gboolean has_timeout;
-	guint id;
+#define NW_GET_NOTIFY_ID(nw) \
+	(GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(nw), "_notify_id")))
+#define NW_GET_NOTIFY_SENDER(nw) \
+	(g_object_get_data(G_OBJECT(nw), "_notify_sender"))
+#define NW_GET_DAEMON(nw) \
+	(g_object_get_data(G_OBJECT(nw), "_notify_daemon"))
 
-	GtkWindow *nw;
+typedef struct
+{
+	NotifyStackLocation type;
+	const gchar *identifier;
+
+} PopupNotifyStackLocation;
+
+const PopupNotifyStackLocation popup_stack_locations[] =
+{
+	{ NOTIFY_STACK_LOCATION_TOP_LEFT,     "top_left"     },
+	{ NOTIFY_STACK_LOCATION_TOP_RIGHT,    "top_right"    },
+	{ NOTIFY_STACK_LOCATION_BOTTOM_LEFT,  "bottom_left"  },
+	{ NOTIFY_STACK_LOCATION_BOTTOM_RIGHT, "bottom_right" },
+	{ NOTIFY_STACK_LOCATION_UNKNOWN,      NULL }
 };
 
-typedef struct _NotifyTimeout NotifyTimeout;
+#define POPUP_STACK_DEFAULT_INDEX 3 /* XXX Hack! */
+
+typedef struct
+{
+	GTimeVal expiration;
+	GTimeVal paused_diff;
+	gboolean has_timeout;
+	gboolean paused;
+	guint id;
+	GtkWindow *nw;
+
+} NotifyTimeout;
 
 struct _NotifyDaemonPrivate
 {
 	guint next_id;
 	guint timeout_source;
 	GHashTable *notification_hash;
-	GSList *poptart_stack;
 	gboolean url_clicked_lock;
+	NotifyStack **stacks;
+	gint stacks_size;
 };
 
 static GConfClient *gconf_client = NULL;
+static DBusConnection *dbus_conn = NULL;
 
 #define CHECK_DBUS_VERSION(major, minor) \
 	(DBUS_MAJOR_VER > (major) || \
@@ -88,8 +116,9 @@ struct _DBusGMethodInvocation
 static void notify_daemon_finalize(GObject *object);
 static void _close_notification(NotifyDaemon *daemon, guint id,
 								gboolean hide_notification);
-static void _emit_closed_signal(GObject *notify_widget);
+static void _emit_closed_signal(GtkWindow *nw);
 static void _action_invoked_cb(GtkWindow *nw, const char *key);
+static NotifyStackLocation get_stack_location_from_string(const char *slocation);
 
 G_DEFINE_TYPE(NotifyDaemon, notify_daemon, G_TYPE_OBJECT);
 
@@ -113,11 +142,35 @@ _notify_timeout_destroy(NotifyTimeout *nt)
 static void
 notify_daemon_init(NotifyDaemon *daemon)
 {
+	NotifyStackLocation location;
+	GConfClient *client = get_gconf_client();
+	GdkDisplay *display;
+	GdkScreen *screen;
+	gchar *slocation;
+	gint i;
+
 	daemon->priv = G_TYPE_INSTANCE_GET_PRIVATE(daemon, NOTIFY_TYPE_DAEMON,
 											   NotifyDaemonPrivate);
 
 	daemon->priv->next_id = 1;
 	daemon->priv->timeout_source = 0;
+
+	slocation = gconf_client_get_string(client, GCONF_KEY_POPUP_LOCATION,
+										NULL);
+	location = get_stack_location_from_string(slocation);
+	g_free(slocation);
+
+	display = gdk_display_get_default();
+	screen = gdk_display_get_default_screen(display);
+	daemon->priv->stacks_size = gdk_screen_get_n_monitors(screen);
+	daemon->priv->stacks = g_new0(NotifyStack *, daemon->priv->stacks_size);
+
+	for (i = 0; i < daemon->priv->stacks_size; i++)
+	{
+		daemon->priv->stacks[i] = notify_stack_new(daemon, screen,
+												   i, location);
+	}
+
 	daemon->priv->notification_hash =
 		g_hash_table_new_full(g_int_hash, g_int_equal, g_free,
 							  (GDestroyNotify)_notify_timeout_destroy);
@@ -126,115 +179,84 @@ notify_daemon_init(NotifyDaemon *daemon)
 static void
 notify_daemon_finalize(GObject *object)
 {
-	NotifyDaemon *daemon;
-	GObjectClass *parent_class;
-
-	daemon = NOTIFY_DAEMON(object);
+	NotifyDaemon *daemon       = NOTIFY_DAEMON(object);
+	GObjectClass *parent_class = G_OBJECT_CLASS(notify_daemon_parent_class);
 
 	g_hash_table_destroy(daemon->priv->notification_hash);
-	g_slist_free(daemon->priv->poptart_stack);
-
-	parent_class = G_OBJECT_CLASS(notify_daemon_parent_class);
+	g_free(daemon->priv);
 
 	if (parent_class->finalize != NULL)
 		parent_class->finalize(object);
 }
 
-NotifyDaemon *
-notify_daemon_new(void)
+static NotifyStackLocation
+get_stack_location_from_string(const char *slocation)
 {
-	return g_object_new(NOTIFY_TYPE_DAEMON, NULL);
+	NotifyStackLocation stack_location = NOTIFY_STACK_LOCATION_DEFAULT;
+
+	if (slocation == NULL || *slocation == '\0')
+		return NOTIFY_STACK_LOCATION_DEFAULT;
+	else
+	{
+		const PopupNotifyStackLocation *l;
+
+		for (l = popup_stack_locations;
+			 l->type != NOTIFY_STACK_LOCATION_UNKNOWN;
+			 l++)
+		{
+			if (!strcmp(slocation, l->identifier))
+				stack_location = l->type;
+		}
+	}
+
+	return stack_location;
+}
+
+static DBusMessage *
+create_signal(GtkWindow *nw, const char *signal_name)
+{
+	guint id = NW_GET_NOTIFY_ID(nw);
+	gchar *dest = NW_GET_NOTIFY_SENDER(nw);
+	DBusMessage *message;
+
+	g_assert(dest != NULL);
+
+	message = dbus_message_new_signal("/org/freedesktop/Notifications",
+									  "org.freedesktop.Notifications",
+									  signal_name);
+
+	dbus_message_set_destination(message, dest);
+	dbus_message_append_args(message,
+							 DBUS_TYPE_UINT32, &id,
+							 DBUS_TYPE_INVALID);
+
+	return message;
 }
 
 static void
 _action_invoked_cb(GtkWindow *nw, const char *key)
 {
-	NotifyDaemon *daemon = g_object_get_data(G_OBJECT(nw), "_notify_daemon");
-	DBusConnection *con;
-	DBusError error;
+	NotifyDaemon *daemon = NW_GET_DAEMON(nw);
+	guint id = NW_GET_NOTIFY_ID(nw);
+	DBusMessage *message;
 
-	dbus_error_init(&error);
+	message = create_signal(nw, "ActionInvoked");
+	dbus_message_append_args(message,
+							 DBUS_TYPE_STRING, &key,
+							 DBUS_TYPE_INVALID);
 
-	con = dbus_bus_get(DBUS_BUS_SESSION, &error);
+	dbus_connection_send(dbus_conn, message, NULL);
+	dbus_message_unref(message);
 
-	if (con == NULL)
-	{
-		g_warning("Error sending ActionInvoked signal: %s", error.message);
-		dbus_error_free(&error);
-	}
-	else
-	{
-		DBusMessage *message;
-		gchar *dest;
-		guint id;
-
-		message = dbus_message_new_signal("/org/freedesktop/Notifications",
-										  "org.freedesktop.Notifications",
-										  "ActionInvoked");
-
-		dest = g_object_get_data(G_OBJECT(nw), "_notify_sender");
-		id = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(nw), "_notify_id"));
-
-		g_assert(dest != NULL);
-
-		dbus_message_set_destination(message, dest);
-		dbus_message_append_args(message,
-								 DBUS_TYPE_UINT32, &id,
-								 DBUS_TYPE_STRING, &key,
-								 DBUS_TYPE_INVALID);
-
-		dbus_connection_send(con, message, NULL);
-
-		dbus_message_unref(message);
-		dbus_connection_unref(con);
-	}
-
-	_close_notification(
-		daemon,
-		GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(nw), "_notify_id")),
-		TRUE);
+	_close_notification(daemon, id, TRUE);
 }
 
 static void
-_emit_closed_signal(GObject *notify_widget)
+_emit_closed_signal(GtkWindow *nw)
 {
-	DBusConnection *con;
-	DBusError error;
-
-	dbus_error_init(&error);
-
-	con = dbus_bus_get(DBUS_BUS_SESSION, &error);
-
-	if (con == NULL)
-	{
-		g_warning("Error sending Close signal: %s", error.message);
-		dbus_error_free(&error);
-	}
-	else
-	{
-		DBusMessage *message;
-		gchar *dest;
-		guint id;
-
-		message = dbus_message_new_signal("/org/freedesktop/Notifications",
-										  "org.freedesktop.Notifications",
-										  "NotificationClosed");
-
-		dest = g_object_get_data(notify_widget, "_notify_sender");
-		id = GPOINTER_TO_UINT(g_object_get_data(notify_widget, "_notify_id"));
-
-		g_assert(dest != NULL);
-
-		dbus_message_set_destination(message, dest);
-		dbus_message_append_args(message,
-								 DBUS_TYPE_UINT32, &id,
-								 DBUS_TYPE_INVALID);
-
-		dbus_connection_send(con, message, NULL);
-
-		dbus_message_unref(message);
-		dbus_connection_unref(con);
-	}
+	DBusMessage *message = create_signal(nw, "NotificationClosed");
+	dbus_connection_send(dbus_conn, message, NULL);
+	dbus_message_unref(message);
 }
 
 static void
@@ -247,7 +269,7 @@ _close_notification(NotifyDaemon *daemon, guint id, gboolean hide_notification)
 
 	if (nt != NULL)
 	{
-		_emit_closed_signal(G_OBJECT(nt->nw));
+		_emit_closed_signal(nt->nw);
 
 		if (hide_notification)
 			theme_hide_notification(nt->nw);
@@ -259,10 +281,51 @@ _close_notification(NotifyDaemon *daemon, guint id, gboolean hide_notification)
 static void
 _notification_destroyed_cb(GtkWindow *nw, NotifyDaemon *daemon)
 {
-	_close_notification(
-		daemon,
-		GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(nw), "_notify_id")),
-		FALSE);
+	_close_notification(daemon, NW_GET_NOTIFY_ID(nw), FALSE);
+}
+
+static void
+_mouse_entered_cb(GtkWindow *nw, GdkEventCrossing *event, NotifyDaemon *daemon)
+{
+	NotifyTimeout *nt;
+	guint id;
+	GTimeVal now;
+
+	if (event->detail == GDK_NOTIFY_INFERIOR)
+		return;
+
+	id = NW_GET_NOTIFY_ID(nw);
+	nt = (NotifyTimeout *)g_hash_table_lookup(daemon->priv->notification_hash,
+											  &id);
+
+	nt->paused = TRUE;
+	g_get_current_time(&now);
+
+	nt->paused_diff.tv_usec = nt->expiration.tv_usec - now.tv_usec;
+	nt->paused_diff.tv_sec  = nt->expiration.tv_sec  - now.tv_sec;
+
+	if (nt->paused_diff.tv_usec < 0)
+	{
+		nt->paused_diff.tv_usec += G_USEC_PER_SEC;
+		nt->paused_diff.tv_sec--;
+	}
+}
+
+static void
+_mouse_exitted_cb(GtkWindow *nw, GdkEventCrossing *event,
+				  NotifyDaemon *daemon)
+{
+	NotifyTimeout *nt;
+	guint id;
+
+	if (event->detail == GDK_NOTIFY_INFERIOR)
+		return;
+
+	id = NW_GET_NOTIFY_ID(nw);
+	nt = (NotifyTimeout *)g_hash_table_lookup(daemon->priv->notification_hash,
+											  &id);
+
+	nt->paused = FALSE;
 }
 
 static gboolean
@@ -286,11 +349,24 @@ _is_expired(gpointer key, gpointer value, gpointer data)
 	if (now_time > expiration_time)
 	{
 		theme_notification_tick(nt->nw, 0);
-		_emit_closed_signal(G_OBJECT(nt->nw));
+		_emit_closed_signal(nt->nw);
 		return TRUE;
 	}
+	else if (nt->paused)
+	{
+		nt->expiration.tv_usec = nt->paused_diff.tv_usec + now.tv_usec;
+		nt->expiration.tv_sec  = nt->paused_diff.tv_sec  + now.tv_sec;
+
+		if (nt->expiration.tv_usec >= G_USEC_PER_SEC)
+		{
+			nt->expiration.tv_usec -= G_USEC_PER_SEC;
+			nt->expiration.tv_sec++;
+		}
+	}
 	else
+	{
 		theme_notification_tick(nt->nw, expiration_time - now_time);
+	}
 
 	*phas_more_timeouts = TRUE;
 
@@ -343,9 +419,8 @@ _calculate_timeout(NotifyDaemon *daemon, NotifyTimeout *nt, int timeout)
 
 		if (daemon->priv->timeout_source == 0)
 		{
-			daemon->priv->timeout_source = g_timeout_add(100,
-														 _check_expiration,
-														 daemon);
+			daemon->priv->timeout_source =
+				g_timeout_add(100, _check_expiration, daemon);
 		}
 	}
 }
@@ -399,13 +474,27 @@ _notify_daemon_process_icon_data(NotifyDaemon *daemon, GtkWindow *nw,
 	GValueArray *image_struct;
 	GValue *value;
 	GArray *tmp_array;
+#if CHECK_DBUS_VERSION(0, 61)
+	GType struct_type;
 
-	if (!G_VALUE_HOLDS(icon_data, G_TYPE_VALUE_ARRAY))
+	struct_type = dbus_g_type_get_struct(
+		"GValueArray",
+		G_TYPE_INT,
+		G_TYPE_INT,
+		G_TYPE_INT,
+		G_TYPE_BOOLEAN,
+		G_TYPE_INT,
+		G_TYPE_INT,
+		dbus_g_type_get_collection("GArray", G_TYPE_UCHAR),
+		G_TYPE_INVALID);
+
+	if (!G_VALUE_HOLDS(icon_data, struct_type))
 	{
 		g_warning("_notify_daemon_process_icon_data expected a "
 				  "GValue of type GValueArray");
 		return FALSE;
 	}
+#endif /* D-BUS >= 0.61 */
 
 	image_struct = (GValueArray *)g_value_get_boxed(icon_data);
 	value = g_value_array_get_nth(image_struct, 0);
@@ -561,151 +650,47 @@ window_clicked_cb(GtkWindow *nw, GdkEventButton *button, NotifyDaemon *daemon)
 	}
 
 	_action_invoked_cb(nw, "default");
-
-	_close_notification(
-		daemon,
-		GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(nw), "_notify_id")),
-		TRUE);
-}
-
-static gboolean
-get_work_area(GtkWidget *nw, GdkRectangle *rect)
-{
-	Atom workarea = XInternAtom(GDK_DISPLAY(), "_NET_WORKAREA", True);
-	Atom type;
-	Window win;
-	int format;
-	gulong num, leftovers;
-	gulong max_len = 4 * 32;
-	guchar *ret_workarea;
-	long *workareas;
-	int result;
-	GdkScreen *screen;
-	int disp_screen;
-
-	gtk_widget_realize(nw);
-	screen = gdk_drawable_get_screen(GDK_DRAWABLE(nw->window));
-	disp_screen = GDK_SCREEN_XNUMBER(screen);
-
-	/* Defaults in case of error */
-	rect->x = 0;
-	rect->y = 0;
-	rect->width = gdk_screen_get_width(screen);
-	rect->height = gdk_screen_get_height(screen);
-
-	if (workarea == None)
-		return FALSE;
-
-	win = XRootWindow(GDK_DISPLAY(), disp_screen);
-	result = XGetWindowProperty(GDK_DISPLAY(), win, workarea, 0,
-								max_len, False, AnyPropertyType,
-								&type, &format, &num, &leftovers,
-								&ret_workarea);
-
-	if (result != Success || type == None || format == 0 || leftovers ||
-		num % 4)
-	{
-		return FALSE;
-	}
-
-	workareas = (long *)ret_workarea;
-	rect->x      = workareas[disp_screen * 4];
-	rect->y      = workareas[disp_screen * 4 + 1];
-	rect->width  = workareas[disp_screen * 4 + 2];
-	rect->height = workareas[disp_screen * 4 + 3];
-
-	XFree(ret_workarea);
-
-	return TRUE;
+	_close_notification(daemon, NW_GET_NOTIFY_ID(nw), TRUE);
 }
 
 static void
-_remove_bubble_from_poptart_stack(GtkWindow *nw, NotifyDaemon *daemon)
+popup_location_changed_cb(GConfClient *client, guint cnxn_id,
+						  GConfEntry *entry, gpointer user_data)
 {
-	NotifyDaemonPrivate *priv = daemon->priv;
-	GdkRectangle workarea;
-	GSList *remove_link = NULL;
-	GSList *link;
-	gint x, y;
+	NotifyDaemon *daemon = (NotifyDaemon*)user_data;
+	NotifyStackLocation stack_location;
+	const char *slocation;
+	GConfValue *value;
+	gint i;
 
-	get_work_area(GTK_WIDGET(nw), &workarea);
+	if (daemon == NULL)
+		return;
 
-	y = workarea.y + workarea.height;
-	x = 0;
+	value = gconf_entry_get_value(entry);
+	slocation = (value != NULL ? gconf_value_get_string(value) : NULL);
 
-	for (link = priv->poptart_stack; link != NULL; link = link->next)
+	if (slocation != NULL && *slocation != '\0')
 	{
-		GtkWindow *nw2 = link->data;
-		GtkRequisition req;
+		stack_location = get_stack_location_from_string(slocation);
+	}
+	else
+	{
+		gconf_client_set_string(get_gconf_client(),
+			"/apps/notification-daemon/popup_location",
+			popup_stack_locations[POPUP_STACK_DEFAULT_INDEX].identifier,
+			NULL);
 
-		if (nw2 != nw)
-		{
-			gtk_widget_size_request(GTK_WIDGET(nw2), &req);
-
-			x = workarea.x + workarea.width - req.width;
-			y = y - req.height;
-
-			theme_move_notification(nw2, x, y);
-		}
-		else
-		{
-			remove_link = link;
-		}
+		stack_location = NOTIFY_STACK_LOCATION_DEFAULT;
 	}
 
-	if (remove_link)
-	{
-		priv->poptart_stack = g_slist_remove_link(priv->poptart_stack,
-												  remove_link);
-	}
-}
-
-static void
-_notify_daemon_add_bubble_to_poptart_stack(NotifyDaemon *daemon,
-										   GtkWindow *nw,
-										   gboolean new_notification)
-{
-	NotifyDaemonPrivate *priv = daemon->priv;
-	GtkRequisition req;
-	GdkRectangle workarea;
-	GSList *link;
-	gint x, y;
-
-	gtk_widget_size_request(GTK_WIDGET(nw), &req);
-
-	get_work_area(GTK_WIDGET(nw), &workarea);
-
-	x = workarea.x + workarea.width - req.width;
-	y = workarea.y + workarea.height - req.height;
-
-	theme_move_notification(nw, x, y);
-
-	for (link = priv->poptart_stack; link != NULL; link = link->next)
-	{
-		GtkWindow *nw2 = GTK_WINDOW(link->data);
-
-		if (nw2 != nw)
-		{
-			gtk_widget_size_request(GTK_WIDGET(nw2), &req);
-			x = workarea.x + workarea.width - req.width;
-			y = y - req.height;
-			theme_move_notification(nw2, x, y);
-		}
-	}
-
-	if (new_notification)
-	{
-		g_signal_connect(G_OBJECT(nw), "destroy",
-						 G_CALLBACK(_remove_bubble_from_poptart_stack),
-						 daemon);
-		priv->poptart_stack = g_slist_prepend(priv->poptart_stack, nw);
-	}
+	for (i = 0; i < daemon->priv->stacks_size; i++)
+		notify_stack_set_location(daemon->priv->stacks[i], stack_location);
 }
 
 static void
 url_clicked_cb(GtkWindow *nw, const char *url)
 {
-	NotifyDaemon *daemon = g_object_get_data(G_OBJECT(nw), "_notify_daemon");
+	NotifyDaemon *daemon = NW_GET_DAEMON(nw);
 	char *escaped_url;
 	char *cmd = NULL;
 
@@ -850,6 +835,8 @@ notify_daemon_notify_handler(NotifyDaemon *daemon,
 
 		if (nt != NULL)
 			nw = nt->nw;
+		else
+			id = 0;
 	}
 
 	if (nw == NULL)
@@ -863,6 +850,10 @@ notify_daemon_notify_handler(NotifyDaemon *daemon,
 						 G_CALLBACK(window_clicked_cb), daemon);
 		g_signal_connect(G_OBJECT(nw), "destroy",
 						 G_CALLBACK(_notification_destroyed_cb), daemon);
+		g_signal_connect(G_OBJECT(nw), "enter-notify-event",
+						 G_CALLBACK(_mouse_entered_cb), daemon);
+		g_signal_connect(G_OBJECT(nw), "leave-notify-event",
+						 G_CALLBACK(_mouse_exitted_cb), daemon);
 	}
 	else
 	{
@@ -939,10 +930,15 @@ notify_daemon_notify_handler(NotifyDaemon *daemon,
 
 			if (icon_info != NULL)
 			{
-				pixbuf = gtk_icon_theme_load_icon(
-					theme, icon,
-					MIN(IMAGE_SIZE, gtk_icon_info_get_base_size(icon_info)),
-					GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
+				gint icon_size = MIN(IMAGE_SIZE,
+									 gtk_icon_info_get_base_size(icon_info));
+
+				if (icon_size == 0)
+					icon_size = IMAGE_SIZE;
+
+				pixbuf = gtk_icon_theme_load_icon(theme, icon, icon_size,
+												  GTK_ICON_LOOKUP_USE_BUILTIN,
+												  NULL);
 
 				gtk_icon_info_free(icon_info);
 			}
@@ -973,9 +969,18 @@ notify_daemon_notify_handler(NotifyDaemon *daemon,
 	}
 	else
 	{
+		gint monitor;
+		GdkScreen *screen;
+		gint x, y;
+
 		theme_set_notification_arrow(nw, FALSE, 0, 0);
-		_notify_daemon_add_bubble_to_poptart_stack(daemon, nw,
-												   new_notification);
+
+		gdk_display_get_pointer(gdk_display_get_default(),
+								&screen, &x, &y, NULL);
+		monitor = gdk_screen_get_monitor_at_point(screen, x, y);
+		g_assert(monitor >= 0 && monitor < priv->stacks_size);
+
+		notify_stack_add_window(priv->stacks[monitor], nw, new_notification);
 	}
 
 	if (!screensaver_active(GTK_WIDGET(nw)) &&
@@ -1008,7 +1013,7 @@ notify_daemon_notify_handler(NotifyDaemon *daemon,
 
 gboolean
 notify_daemon_close_notification_handler(NotifyDaemon *daemon,
-										 guint id, GError ** error)
+										 guint id, GError **error)
 {
 	_close_notification(daemon, id, TRUE);
 
@@ -1066,10 +1071,11 @@ main(int argc, char **argv)
 	gconf_init(argc, argv, NULL);
 
 	gconf_client = gconf_client_get_default();
-	gconf_client_add_dir(gconf_client, "/apps/notification-daemon/theme",
+	gconf_client_add_dir(gconf_client, GCONF_KEY_DAEMON,
 						 GCONF_CLIENT_PRELOAD_NONE, NULL);
 
 	error = NULL;
+
 	connection = dbus_g_bus_get(DBUS_BUS_SESSION, &error);
 
 	if (connection == NULL)
@@ -1080,6 +1086,8 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
+	dbus_conn = dbus_g_connection_get_connection(connection);
+
 	dbus_g_object_type_install_info(NOTIFY_TYPE_DAEMON,
 									&dbus_glib__object_info);
 
@@ -1098,7 +1106,14 @@ main(int argc, char **argv)
 		g_error("Could not aquire name: %s", error->message);
 	}
 
-	daemon = notify_daemon_new();
+	daemon = g_object_new(NOTIFY_TYPE_DAEMON, NULL);
+
+	gconf_client_notify_add(gconf_client, GCONF_KEY_POPUP_LOCATION,
+							popup_location_changed_cb, daemon,
+							NULL, NULL);
+
+	/* Emit signal to verify/set current key */
+	gconf_client_notify(gconf_client, GCONF_KEY_POPUP_LOCATION);
 
 	dbus_g_connection_register_g_object(connection,
 										"/org/freedesktop/Notifications",
